import java.net.*;
import java.io.*;
import java.util.*;

/**
 * This class servers one chat client.
 * @author Mika Vesterholm
 * @version 1.0
 */
public class ClientHandler extends Thread {
  /** Vector containing all active ClientHandlers. */
  Vector others;
  /** The socket. */
  Socket socket;
  /** Writer to do the writing. */
  BufferedWriter writer;
  /** Reader to do the reading. */
  BufferedReader reader;
  /** Users name. Null if user is not known. */
  String userName = null;

  /**
   * Constructs client handler.
   * @param s the socket
   * @param others Vector containing all active ClientHandlers.
   */
  public ClientHandler(Socket s, Vector others) {
    socket = s;
    this.others = others;
  }

  /**
   * The run loop serves the client. Reads clients request and
   * does what is needed.
   */
  public void run() {
    try {
      writer = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream(),"UTF-8"));
      reader = new BufferedReader(new InputStreamReader(socket.getInputStream(),"UTF-8"));

      String line = reader.readLine();
      if (line == null || !line.equals("ChatClient ver1.0") )
        return;

      userName = reader.readLine();
      if ( userName == null  )
        return;

      writer.write("OK\n");
      writer.flush();

      others.addElement(this);

      while (true) {
        line = reader.readLine();
        if (line == null) return;
        if (line.equals("Exit")) return;
        if (line.equals("Message")) processMessage();
      }
    } catch (Exception ex) {
      ex.printStackTrace();
    } finally {
      others.removeElement(this);
    }

    try {
      socket.close();
    } catch (Exception ex) {
    }
  }

  /**
   * Reads message sent by the client and sends it to other clients.
   * @throws Exception if fails to read message.
   */
  private void processMessage() throws Exception {
    String message = reader.readLine();
    if (message == null) throw new Exception("EOF");
    Vector to = (Vector)others.clone();
    int size = to.size();
    for(int i=0; i<size; i++) {
      ClientHandler h = (ClientHandler)to.elementAt(i);
      if (h == this) continue;
      h.sendMessage(userName,message);
    }
  }

  /**
   * Sends message to this client. Method closes the connection if
   * sending fails. <B>This method is synchronized</B> so it is safe
   * to call it from multiple threads.
   * @param sender the sender of the message.
   * @param message message to send.
   */
  public synchronized void sendMessage(String sender, String message) {
    if ( writer == null ) return;
    try {
      writer.write("Message\n");
      writer.write(sender+"\n");
      writer.write(message+"\n");
      writer.flush();
    } catch (Exception ex) {
      ex.printStackTrace();
      try {
        socket.close();
      } catch (Exception e) {
      }
    }
  }
}